# -*- coding: utf-8 -*-

REQ_SERVER_INFO = "server"

REQ_MISSION_STATUS = "mission"
REQ_MISSION_LOAD = "mission LOAD {0}"
REQ_MISSION_BEGIN = "mission BEGIN"
REQ_MISSION_END = "mission END"
REQ_MISSION_DESTROY = "mission DESTROY"
